-- region 音效管理器
-- Date 2017/11/24
-- Author : Edward
-- 此文件由[BabeLua]插件自动生成

require "common/class"
require "common/define"

AudioManager = Class()

function AudioManager:ctor()
    self.soundsList = {} -- 音效列表
    self.bgMusic = {}-- 背景音乐列表
    self.musicPlayer = nil
    self.soundPlayer = nil
    self.curMusic = ""
    self.musicVolume = 0.2 -- 音量大小
    self.soundVolume = 1
    self.musicSwitch = true -- 开关
    self.soundSwitch = true

    self.soundsLen = 0
end

function AudioManager:init()
    self.musicSwitch = PlayerPrefs.GetInt(gameDef.MusicSwitchKey, 1) == 1
    self.soundSwitch = PlayerPrefs.GetInt(gameDef.SoundSwitchKey, 1) == 1

    self:initMusic()
    self:initSound()
    self.soundsList = {}
    self.bgMusic = {}

    self:setMusicSwitch(self.musicSwitch)
    self:setSoundSwitch(self.soundSwitch)
end

-- 初始化背景音乐节点
function AudioManager:initMusic()
    local name = "MusicPlayer"
    self.musicPlayer = GameObject.Find(name)
    if self.musicPlayer == nil then
        self.musicPlayer = GameObject.New(name)
        self.musicPlayer.name = name
        self.musicPlayer:AddComponent(typeof(AudioSource))
    end
    GameObject.DontDestroyOnLoad(self.musicPlayer)
end

-- 初始化音效节点
function AudioManager:initSound()
    local name = "SoundPlayer"
    self.soundPlayer = GameObject.Find(name)
    if self.soundPlayer == nil then
        self.soundPlayer = GameObject.New(name)
        self.soundPlayer.name = name
    end
    GameObject.DontDestroyOnLoad(self.soundPlayer)
end

-- 播放背景音乐
function AudioManager:playMusic(name, isLoop)
    if name ~= self.curMusic then
        local ac = self.bgMusic[name]
        if ac == nil then
            ac = resMgr:loadObject(gameDef.ResMusicPrefix .. name, name)
        end
        if ac ~= nil then
            if self.curMusic ~= nil and self.curMusic ~= "" then
                resMgr:unloadAssetBundle(gameDef.ResMusicPrefix .. self.curMusic, self.curMusic)
            end
            self.curMusic = name
            local audioSource = self.musicPlayer:GetComponent("AudioSource")
            audioSource.clip = ac
            audioSource.loop = isLoop
            audioSource.volume = self.musicVolume
            audioSource:Play()
        end
    end
end

-- 播放音效
function AudioManager:playSound(name, force)
    if name == nil or name == "" then
        return
    end
    force = force or false
    if force then
--        local len = #self.soundsList
--        for i = len, 1, -1 do
--            if self.soundsList[i] ~= nil then
--                resMgr:unloadAssetBundle(gameDef.ResSoundPrefix .. name)
--                GameObject.Destroy(self.soundsList[i].gameObject)
--                table.remove(self.soundsList, i)
--            end
--        end
        self:destroyAllSounds()
    end
    local ac = resMgr:loadObject(gameDef.ResSoundPrefix .. name, name)
    if ac ~= nil then
        local gObject = GameObject.New()
        gObject.name = name
        gObject.transform:SetParent(self.soundPlayer.transform)
        local audioSource = gObject:AddComponent(typeof(AudioSource))
        audioSource.clip = ac
        audioSource.volume = self.soundVolume
        audioSource:Play()
        table.insert(self.soundsList, audioSource)
    end
end

function AudioManager:destroyAllSounds()
    local len = #self.soundsList
    for i = len, 1, -1 do
        if self.soundsList[i] ~= nil then
            resMgr:unloadAssetBundle(gameDef.ResSoundPrefix .. self.soundsList[i].gameObject.name)
            GameObject.Destroy(self.soundsList[i].gameObject)
            table.remove(self.soundsList, i)
        end
    end
end

function AudioManager:setMusicSwitch(state)
    self.musicSwitch = state
    self.musicVolume = state and 0.2 or 0
    local audioSource = self.musicPlayer:GetComponent("AudioSource")
    audioSource.volume = self.musicVolume
end

function AudioManager:setSoundSwitch(state)
    self.soundSwitch = state
    self.soundVolume = state and 1 or 0
end

function AudioManager:isMusicOn()
    return self.musicSwitch
end

function AudioManager:isSoundOn()
    return self.soundSwitch
end

function AudioManager:destroySound(obj)
    resMgr:unloadAssetBundle(gameDef.ResSoundPrefix .. obj.name)
    GameObject.Destroy(obj)
end

function AudioManager:update()
    self.soundsLen = #self.soundsList
    local audioSource = nil
    for i = self.soundsLen, 1, -1 do
        audioSource = self.soundsList[i]
        if audioSource ~= nil and audioSource.gameObject ~= nil and not audioSource.isPlaying then
            coroutine.start(self.destroySound, self, audioSource.gameObject)
            table.remove(self.soundsList, i)
        end
    end
end

-- endregion
