--[[
	协议管理器
	@Liukeming 2017-11-30
	
	用于在NetworkManager和逻辑代码之间建立调用的桥梁
	协议经过打包后生成pb_xxx.lua文件，全部被protocl/pb_all所引用
     
	因此发送时，只需要在逻辑代码中编写（以一个带数组的复合对象为例）
		local msg = protocol.Equip()
		msg.id = 1
		msg.name = 'xxx'
		msg.complexs[1] = protocol.Complex()
		msg.complexs[2] = protocol.Complex()
		msg.send()
	
	在msg.send中自然会调用ProtocolManager.send(BytesBuffer)
	然后在prototype
]]

local M = {}
_G.ProtocolManager = M
_G.protocol = {}
M.uniqueId = 0
M.communiCallBack = {}

--由PB对象调用
function M.send(msgObj)
    msgObj:send()
end

--由C#层网络管理器调用，参数是BytesBuffer对象
function --[[private]] M.recieve(buffer)
	local id = buffer:ReadShort()--ID定义为数字或字符串字符串都行
    local uniqueId = 0--buffer:ReadInt()
    local invalid2 = 0--buffer:ReadShort()

    --大于40000为广播消息
    if id >= 30000 then
        
    else
        uniqueId = buffer:ReadInt()
        invalid2 = buffer:ReadShort()
    end
    local iType = buffer:ReadShort()

	local msg = protocol[id + iType]
	if not msg then
		_gameTools.luaError("未定义协议：  "..id.. " itype = " .. iType .. " " ..(id + iType))
        return
	end

    --_gameTools.luaError("enter decode id=" .. id..' itype='..iType..' '..(id + iType))
    if msg.decode ~= nil then
        if (id + iType) == 1001 then
            --如果是登录操作额外处理
            local tempData = {}
            tempData.result = buffer:ReadShort()
            if tempData.result == 118 then
                tempData.reason = buffer:ReadUTF()
                tempData.time = buffer:ReadInt()
                ProtocolManager.broadcast(uniqueId, tempData)
            elseif tempData.result == 11 then
                ProtocolManager.broadcast(uniqueId, tempData)
            elseif tempData.result == 0 then
                tempData.player_bytesWriteFore = p_player_bytesWriteFore_read(buffer)
	            ProtocolManager.broadcast(uniqueId, tempData)
            else
                _gameTools.luaError("未处理的错误码: "..tempData.result)
            end
        elseif (id + iType) == 1005 then
            --如果是重连操作额外处理
            local tempData = {}
            tempData.result = buffer:ReadShort()
            if tempData.result == 118 then
                tempData.reason = buffer:ReadUTF()
                tempData.time = buffer:ReadInt()
                ProtocolManager.broadcast(uniqueId, tempData)
            elseif tempData.result == 0 then
                tempData.player_bytesWriteFore = p_player_bytesWriteFore_read(buffer)
	            ProtocolManager.broadcast(uniqueId, tempData)
            end
        elseif (id + iType) == 7201 then
            --聊天 GM工具控制禁言需要特殊处理
            local tempData = {}
            tempData.result = buffer:ReadShort()
            if tempData.result == 34 then
                tempData.reason = buffer:ReadUTF()
                tempData.time = buffer:ReadInt()
                ProtocolManager.broadcast(uniqueId, tempData)
            elseif tempData.result == 0 then
	            ProtocolManager.broadcast(uniqueId, tempData)
            end
        else
            msg.decode(uniqueId, buffer)
        end
    else
        _gameTools.luaError("协议 "..id .. " 没有decode函数")
    end
end

function M.register(protoId, func)
    if protocol == nil then protocol = {} end
    protocol[protoId] = func
end

--获取唯一id
function M.getUniqueId()
    M.uniqueId = M.uniqueId + 1
    return M.uniqueId
end

function M.addListener(uniqueId, tab1, successCallBack, tab2, failedCallBack)
    local t = {}
    t.tab1 = tab1
    t.successCallBack = successCallBack
    t.tab2 = tab2
    t.failedCallBack = failedCallBack
    t.uniqueId = uniqueId
    M.communiCallBack[uniqueId] = t
end

function M.broadcast(uniqueId, data)
    _gameTools.printMsg(uniqueId , data)
    _gameTools.closeConnnecting(uniqueId)

    local t = M.communiCallBack[uniqueId]
    if data.result == 0 then
        if t.tab1 ~= nil then
            t.successCallBack(t.tab1, data)
        else
            t.successCallBack(data)
        end
    else
        if t.failedCallBack == nil then
            M.failedFuncCommon(data)
        else
            if t.tab2 ~= nil then
                t.failedCallBack(t.tab2, data)
            else
                t.failedCallBack(data)
            end
        end
    end
    M.communiCallBack[uniqueId] = nil

    --@Liukeming 2018-1-4 收到消息就刷新可激活的引导
    gameDataManager.guideData:checkActivable()
end

function M.failedFuncCommon(data)
    if data.result == 110 then
        eventMgr:broadcastEvent(_EventType.Menke_LevelUpBookFail, false)
        return
    end

    gameDef.IsPlayItemGetEffect = true

    _Class[uiPopupWarning].show((t_errorcode[data.result] and t_errorcode[data.result].desc or ("未定义提示："..data.result)))
end

--require "message/pb_all"