--region 资源管理器
--Date 2017/11/24
--Author : Edward
--此文件由[BabeLua]插件自动生成

require "common/class"

--资源类型
_EAssetType = 
{
    None = 0,
    UI = 1,
    GameObject = 2,
    Texture = 3,
    Scene = 4,
    Sprite = 5,
}

ResourceManager = Class()

function ResourceManager:ctor()
    self.assetBundleManifest = nil
    self.mapGameAssets = {}         --存放当前加载好的资源列表
    self.delMapGameAssets = {}      --要删除的资源列表
    self.isDeleting = false         --是否正在删除资源

    self.downloadList = {}          --加载列表
    self.curDownloadTask = nil      --当前正在执行的加载任务
end

function ResourceManager:init()
    if self.assetBundleManifest ~= nil then
        return
    end
    local ab = AssetBundle.LoadFromFile(gameDef.ManifestPath)
    self.assetBundleManifest = ab:LoadAsset("AssetBundleManifest")
end

function ResourceManager:update()
     if self.curDownloadTask == nil and #self.downloadList > 0 then
        self.curDownloadTask = self.downloadList[1]
        table.remove(self.downloadList, 1)
        self:loadResByAsync(self.curDownloadTask)
     end

     if not self.isDeleting and _gameTools.getTableCount(self.delMapGameAssets) > 0 then
        self.isDeleting = true
        coroutine.start(self.unloadDelAB, self)
     end
end

--region 一般用作类内部调用

--同步加载,直接加载,不放入等待队列
function ResourceManager:synchroLoad(loadInfo)
    self:loadDependencies(loadInfo.assetBundleName, true)
    local asset = self:getAssetDontAddRef(loadInfo.assetBundleName)
    if asset == nil then
        asset = self.delMapGameAssets[loadInfo.assetBundleName]
        --判断删除列表里面是否存在
        if asset ~= nil then
            self.delMapGameAssets[loadInfo.assetBundleName] = nil
            asset.assetName = loadInfo.resName
            asset.assetType = loadInfo.assetType
            self.mapGameAssets[asset.assetBundleName] = asset
        else
            self:loadResFromFile(loadInfo)
        end
    end
    self:getAssetObj(loadInfo)
end

--异步加载,放入加载队列
function ResourceManager:asynchroLoad(loadInfo)
    self:loadDependencies(loadInfo.assetBundleName, false)
    local asset = self:getAssetDontAddRef(loadInfo.assetBundleName)
    if asset == nil then
        --判断删除列表里面是否存在
        asset = self.delMapGameAssets[loadInfo.assetBundleName]
        if asset ~= nil then
            self.delMapGameAssets[loadInfo.assetBundleName] = nil
            asset.assetName = loadInfo.resName
            asset.assetType = loadInfo.assetType
            self.mapGameAssets[asset.assetBundleName] = asset
            self:getAssetObj(loadInfo)
        else
            table.insert(self.downloadList, loadInfo)
        end
    else
        self:getAssetObj(loadInfo)
    end
end

--加载依赖资源
function ResourceManager:loadDependencies(assetBundleName, isSync)
    local dependencies = self.assetBundleManifest:GetAllDependencies(assetBundleName)
    if dependencies.Length == 0 then return end
    for i = 1, dependencies.Length do
        local value = dependencies:GetValue(i - 1)
        local loadInfo = DownloadInfo.new()
        loadInfo.assetBundleName = value
        loadInfo.resPath = gameDef.ResRootPath .. value
        loadInfo.assetType = _EAssetType.None
        if isSync then
            self:synchroLoad(loadInfo)
        else
            self:asynchroLoad(loadInfo)
        end
    end
end

--获取AB上的Obj
function ResourceManager:getAssetObj(loadInfo)
    local asset = self:getAssetAddRef(loadInfo.assetBundleName)
    if asset ~= nil and loadInfo.loadCompleted ~= nil then
        if loadInfo.callBackTable ~= nil then
            loadInfo.loadCompleted(loadInfo.callBackTable, asset.assetName, asset.assetBundle, asset.assetType, loadInfo.otherInfo)
        else
            loadInfo.loadCompleted(asset.assetName, asset.assetBundle, asset.assetType, loadInfo.otherInfo)
        end
    end
end

--AssetBundle的同步加载方式
function ResourceManager:loadResFromFile(loadInfo)
    local ab = AssetBundle.LoadFromFile(loadInfo.resPath)
    local asset = GameAsset.new()
    asset.assetName = loadInfo.resName
    asset.assetBundleName = loadInfo.assetBundleName
    asset.assetBundle = ab
    asset.assetType = loadInfo.assetType
    self.mapGameAssets[asset.assetBundleName] = asset
end

--通过WWW进行异步加载资源
function ResourceManager:loadResByAsync(loadInfo)
    local asset = self:getAssetDontAddRef(loadInfo.assetBundleName)
    if asset == nil then
        CGameUtil.StartLoadResByAsync(loadInfo.resPath, self.asyncLoadCompeleted, self)
    else
        self:getAssetObj(loadInfo)
        self.curDownloadTask = nil
    end
end

--异步加载完成回调
function ResourceManager:asyncLoadCompeleted(assetBundleRequest)
    local loadInfo = self.curDownloadTask
    if assetBundleRequest.assetBundle ~= nil then
        local asset = GameAsset.new()
        asset.assetName = loadInfo.resName
        asset.assetBundleName = loadInfo.assetBundleName
        asset.assetBundle = assetBundleRequest.assetBundle
        asset.assetType = loadInfo.assetType
        self.mapGameAssets[asset.assetBundleName] = asset
        self:getAssetObj(loadInfo)
    else
        _gameTools.luaError(string.format("异步加载 %s 失败", loadInfo.assetBundleName))
    end
    self.curDownloadTask = nil
end

--判断在等待列表中是否包含这个资源
function ResourceManager:isContainsInDownloadList(loadInfo)
    for i = 1, #self.downloadList do
        if self.downloadList[i].assetBundleName == loadInfo.assetBundleName then
            return true
        end
    end
    return false
end

--获取asset不添加引用次数
function ResourceManager:getAssetDontAddRef(abName)
    local asset = self.mapGameAssets[abName]
    return asset
end

--获取GameAsset并添加引用次数
function ResourceManager:getAssetAddRef(abName)
    local asset = self.mapGameAssets[abName]
    if asset ~= nil then
        asset:addRef()
        return asset
    end
    return nil
end

--减引用次数
function ResourceManager:subAssetRef(abName)
    local asset = self.mapGameAssets[abName]
    if asset ~= nil then
        asset:subRef()
        if asset.refCount <= 0 then
            self:unloadAssetBundle(abName)
        end
    else
        _gameTools.luaWarning("subAssetRef : 资源--> "..name.."  不存在")
    end
end

--通过名字卸载资源, 
--abName : 资源名字
function ResourceManager:unloadAssetBundle(abName)
    abName = string.lower(abName)
    local asset = self:getAssetDontAddRef(abName)
    if asset == nil then 
        _gameTools.luaWarning(string.format("资源 : %s 不存在",abName))
        return 
    end

    self:unloadDependencies(asset.assetBundleName)
    if asset:subRef() <= 0 then
        self.delMapGameAssets[abName] = asset
        self.mapGameAssets[abName] = nil
    end
end

--卸载依赖资源
function ResourceManager:unloadDependencies(assetBundleName, isSync)
    local dependencies = self.assetBundleManifest:GetAllDependencies(assetBundleName)
    if dependencies.Length == 0 then return end
    for i = 1, dependencies.Length do
        local value = dependencies:GetValue(i - 1)
        self:unloadAssetBundle(value)
    end
end

--卸载要删除的资源资源
function ResourceManager:unloadDelAB()
    local asset = nil
    local delList = {}
    for k, v in pairs(self.delMapGameAssets) do
        _gameTools.luaWarning("unload Ab : "..k)
        v:unloadAssetBundle()
        table.insert(delList, k)
    end
    for i = 1, #delList do
        self.delMapGameAssets[delList[i]] = nil
    end
    self.isDeleting = false
end

--endregion


--region 供外部接口

--加载资源，是否同步
function ResourceManager:loadResObj(loadInfo, isAsync)
    if isAsync then
        self:asynchroLoad(loadInfo)
    else
        self:synchroLoad(loadInfo)
    end
end

function ResourceManager:loadGameObject(abName)
    
end

function ResourceManager:loadSprite(abName, spriteName)
    local loadInfo = DownloadInfo.new()
    loadInfo.assetBundleName = string.lower(abName)
    loadInfo.resPath = gameDef.ResRootPath .. loadInfo.assetBundleName
    self:synchroLoad(loadInfo)
    local asset = self:getAssetDontAddRef(loadInfo.assetBundleName)
    return CGameUtil.TransToSprite(asset.assetBundle:LoadAsset(spriteName))
end

function ResourceManager:loadAudio(abName, audioName)
    local loadInfo = DownloadInfo.new()
    loadInfo.assetBundleName = string.lower(abName)
    loadInfo.resPath = gameDef.ResRootPath .. loadInfo.assetBundleName
end

function ResourceManager:loadObject(abName, assetName)
    local loadInfo = DownloadInfo.new()
    loadInfo.assetBundleName = string.lower(abName)
    loadInfo.resPath = gameDef.ResRootPath .. loadInfo.assetBundleName
    self:synchroLoad(loadInfo)
    local asset = self:getAssetDontAddRef(loadInfo.assetBundleName)
    return asset.assetBundle:LoadAsset(assetName)
end

function ResourceManager:loadAb(abName)
    local loadInfo = DownloadInfo.new()
    loadInfo.assetBundleName = string.lower(abName)
    loadInfo.resPath = gameDef.ResRootPath .. loadInfo.assetBundleName
    self:synchroLoad(loadInfo)
    local asset = self:getAssetDontAddRef(loadInfo.assetBundleName)
    return asset
end

--endregion



-------------------资源类------------------
GameAsset = Class()

function GameAsset:ctor()
    self.assetBundleName = ""   --assetBundleName
    self.assetName = ""         --资源名字
    self.assetObj = nil         
    self.assetBundle = nil      --assetBundle
    self.assetType = _EAssetType.None       --资源类型
    self.refCount = 0       --引用计数
end

--添加引用
function GameAsset:addRef()
    self.refCount = self.refCount + 1
    return self.refCount
end

--减引用
function GameAsset:subRef()
    self.refCount = self.refCount - 1
    return self.refCount
end

--卸载资源
function GameAsset:unloadAssetBundle()
    if self.assetBundle ~= nil then
        self.assetBundle:Unload(true)
        self.assetBundle = nil
    end
end

------------------------- 加载资源信息类 ---------------------------------
DownloadInfo = Class()

function DownloadInfo:ctor()
    self.assetBundleName = ""           --assetBundleName
    self.resName = ""                   --资源名字
    self.resPath = ""                   --资源路径
    self.assetType = _EAssetType.None   --资源类型
    self.assetBundle = nil              --AssetBundle
    self.loadCompleted = nil            --加载完成后的回调
    self.callBackTable = nil            --回调对象table
    self.otherInfo = nil                --其他信息
end

----------------------预加载类型，主要用于uiLoading界面设置预加载资源------------------
PreLoadInfo = Class()

function PreLoadInfo:ctor(assetType, assetName)
    self.assetType = assetType
    self.assetName = assetName
    self.doFunc = nil
    self.doFuncArgs = nil
    self.callBack = nil
    self.callBackTable = nil
    self.callBackArgs = nil
end

--endregion
