--region ui管理器
--Date 2017/11/24
--Author : Edward
--此文件由[BabeLua]插件自动生成

require "common/class"
require "common/define"
require "ui/UIConfig"

UIManager = Class()


function UIManager:ctor()
    self.curLayer = 0       --当前打开UI的层级
    self.uiCanvas = nil
    self.normalLayer = nil
    self.popupLayer = nil
    self.guideLayer = nil
    self.dialogLayer = nil
    self.fixedLayer = nil
    self.openUIList = {}
    self.closeUIList = {}
end

-- 初始化
function UIManager:init()
    --初始化所有ui脚本
    self.uiContainer = _UIConfig

    local uiRoot = GameObject.Find("uiRoot")
    if uiRoot == nil then
        uiRoot = CGameUtil.ObjToGameObject(GameObject.Instantiate(Resources.Load("uiRoot")))
    end
    GameObject.DontDestroyOnLoad(uiRoot)
    self.uiCanvas = uiRoot.transform:Find("staticCanvas")

    self.normalLayer = self:addLayer(_UILayerType.Normal)
    self.popupLayer = self:addLayer(_UILayerType.Popup)
    self.dialogLayer = self:addLayer(_UILayerType.Dialog)
    self.fixedLayer = self:addLayer(_UILayerType.Fixed)
end

--添加layer节点
function UIManager:addLayer(name, siblingIndex)
    local layerObj = self.uiCanvas:Find(name)
    if layerObj == nil then
        layerObj = GameObject.New(name).transform
        layerObj:SetParent(self.uiCanvas)
        layerObj.localPosition = Vector3.zero
        layerObj.localScale = Vector3.one
    end
    if siblingIndex ~= nil then
        layerObj:SetSiblingIndex(siblingIndex)
    end
    return layerObj
end

--打开界面
function UIManager:openUI(uiName, bIsAsync, ...)
    if self:isExistOpenList(uiName) then
        _gameTools.luaError("openUI -> ".. uiName.. " was be opened")
        return
    end
    self:loadUI(uiName, bIsAsync, ...)
end

--关闭UI
function UIManager:closeUI(uiName)
    local ui, index = self:isExistOpenList(uiName)
    if ui ~= nil then
        table.remove(self.openUIList, index)
        ui:close()
        local uiConfig = _UIConfig[uiName]
        resMgr:unloadAssetBundle(string.lower(uiConfig.resName))
        eventMgr:broadcastEvent(_EventType.CheckCacheGuide)
    else
        _gameTools.luaError("closeUI -> "..uiName.." was closed")
    end
end

--获取当前正打开的UI面板
function UIManager:getCurUI()
    return self.openUIList[#self.openUIList]
end

--获取当前顶层UI,此接口会过滤掉忽略管理的UI
function UIManager:getTopUI()
    for i = #self.openUIList, 1, -1 do
        local uiConfig = _UIConfig[self.openUIList[i].uiName]
        if not uiConfig.ignoreMgr then
            return self.openUIList[i]
        end
    end
    return nil
end

function UIManager:layerToNumber(layer1)
    if layer1 == _UILayerType.Normal then
        return 1
    elseif layer1 == _UILayerType.Popup then
        return 2
    elseif layer1 == _UILayerType.Guide then
        return 3
    elseif layer1 == _UILayerType.Dialog then
        return 4
    elseif layer1 == _UILayerType.Fixed then
        return 5
    else
        return 1
    end
end

--加载UI
function UIManager:loadUI(uiName, bIsAsync, ...)
	local uiInfo = _UIConfig[uiName]
    local loadInfo = DownloadInfo.new()
	loadInfo.resName = uiInfo.resName
    loadInfo.assetBundleName = gameDef.ResUIPrefix .. string.lower(uiInfo.resName)
    loadInfo.resPath = gameDef.ResRootPath .. loadInfo.assetBundleName
    loadInfo.assetType = _EAssetType.UI
    loadInfo.loadCompleted = self.loadCompleted
    loadInfo.callBackTable = self
    loadInfo.otherInfo = {}
    loadInfo.otherInfo.uiName = uiName
    loadInfo.otherInfo.args = {...}
    resMgr:loadResObj(loadInfo, bIsAsync)
end

--是否存在于open列表中
function UIManager:isExistOpenList(uiName)
    for i = 1, #self.openUIList do
        if self.openUIList[i].uiName == uiName then
            return self.openUIList[i], i
        end
    end
    return nil
end

--获取ui类型
function UIManager:getUILayerType(uiName)
    return _UIConfig[uiName].layer
end

--设置界面的root节点
function UIManager:setPanelRoot(otherInfo, uiGo)
    local uiType = _UIConfig[otherInfo.uiName].layer
    local parentRoot = self.normalLayer
    if uiType == _UILayerType.Popup then
        parentRoot = self.popupLayer
    elseif uiType == _UILayerType.Dialog then
        parentRoot = self.dialogLayer
    elseif uiType == _UILayerType.Fixed then
        parentRoot = self.fixedLayer
    end
    uiGo.transform:SetParent(parentRoot)
    uiGo.transform.localPosition = Vector3.zero
    uiGo.transform.localScale = Vector3.one
    local rectTrans = uiGo:GetComponent(_UICom.RectTransform)
    rectTrans:SetInsetAndSizeFromParentEdge(RectTransform.Edge.Left, 0, 0)
    rectTrans:SetInsetAndSizeFromParentEdge(RectTransform.Edge.Top, 0, 0)
    rectTrans.anchorMin = Vector2.zero
    rectTrans.anchorMax = Vector2.one
end

--初始化某个界面
function UIManager:initPanel(otherInfo, uiGo)
    local uiCom = uiGo:AddComponent(typeof(CUIBase))
    local uiLua = _UIConfig[otherInfo.uiName].lua
    uiLua.uiObject = uiGo
    uiLua.uiState = _EUIState.Open
    uiLua.script = uiCom
    uiLua.uiName = otherInfo.uiName
    uiCom.luaTab = uiLua
    uiCom:SetOpenCallArgs(otherInfo.args)
    table.insert(self.openUIList, uiLua)
end

function UIManager:loadCompleted(resName, assetBundle, assetType, otherInfo)
    if assetType == _EAssetType.UI then
        --管理
        local uiGo = GameObject.Instantiate(assetBundle:LoadAsset(resName))      
        self:setPanelRoot(otherInfo, uiGo)     --设置layer节点
        self:initPanel(otherInfo, uiGo)
        eventMgr:broadcastEvent(_EventType.PreLoadCallBack)
        eventMgr:broadcastEvent(_EventType.CacheGuide)
    elseif assetType == _EAssetType.Font then
        
    end
end

function UIManager:getUI(uiName)
    return _UIConfig[uiName].lua
end

--更新逻辑
function UIManager:update()
--    for k, v in pairs(self.openUIList) do
--        if v and v.update ~= nil then
--            v:update()
--        end
--    end
end

function UIManager:clear()
    for i = #self.openUIList, 1, -1 do
        local uiConfig = _UIConfig[self.openUIList[i].uiName]
        if not uiConfig.ignoreMgr then
            self:closeUI(self.openUIList[i].uiName)
        end
    end
end