--[[
    @Liukeming 2018-1-3
    条件管理器，专门负责检测各类条件是否满足参数
    这个不走gameDataManager，它是一堆函数的集合，不是动态数据
]]

--命令类型
CommandType = 
{
    Condition, --条件判断
    Operation, --操作类型
}
--[[
    命令配置，表里面个格式为 【命令名称】+ 【操作符】 + 【参数1，参数2 。。。】
    例如
    guests>10
    open=WINDOW1
    position+=100
    解析后的操作符会作为args的第一个元素
    比如guests>10就会变成 {>,10}传到callback里面
]]
CommandConfig = 
{
    --===============条件定义
    --等级
    Level =
    {
        id = 1,
        name = 'level',
        type = CommandType.Condition,
        callback = function(args)
            return CommandCompare(gameDataManager.playerInfo.level, args[2], args[1])
        end
    },
    --宾客数量
    Guests = 
    {
        id = 2,
        name = 'guests',
        type = CommandType.Condition,
        callback = function(args)
            return CommandCompare(#gameDataManager.menkeData:getMenkeList(), args[2], args[1])
        end
    },
    --红颜数量
    Wifes = 
    {
        id = 3,
        name = 'wifes',
        type = CommandType.Condition,
        callback = function(args)
            return CommandCompare(#gameDataManager.getWifeInfo(), args[2], args[1])
        end
    },
    --孩子数量
    Babys = 
    {
        id = 4,
        name = 'babys',
        type = CommandType.Condition,
        callback = function(args)
            return CommandCompare(#gameDataManager.getAllChildrenListData(), args[2], args[1])
        end
    },
    --成年孩子数量
    Adults = 
    {
        id = 5,
        name = 'adults',
        type = CommandType.Condition,
        callback = function(args)
            return CommandCompare(#gameDataManager.getKejuChildrenListData(), args[2], args[1])
        end
    },
    --囚犯数量
    Prisoners = 
    {
        id = 6,
        name = 'prisoners',
        type = CommandType.Condition,
        callback = function(args)
            print('牢房犯人数量（用于新手引导测试）', #gameDataManager.playerInfo.laofangData.prisoners)
            return CommandCompare(#gameDataManager.playerInfo.laofangData.prisoners,args[2], args[1])
        end
    },
    --当前窗口
    CurrentWindow = 
    {
        id = 7,
        name = 'window',
        type = CommandType.Condition,
        callback = function(args)
            local ui = uiMgr:getTopUI()
            return ui.uiName == args[2] and ui._started==true
        end
    },

    --特殊判断条件
    If = 
    {
        id = 99,
        name = 'if',
        type = CommandType.Condition,
        callback = function(args)
            local cmd = args[2]

            --府邸没有别的窗口打开
            if cmd == 'isCityClear' then
                local wnd = uiMgr:getTopUI()
                return wnd._started and wnd.uiName == 'UIMainCity'
            elseif cmd == 'isLianyinClear' then
                local wnd = uiMgr:getTopUI()
                if not wnd._started or wnd.uiName ~= 'UILianYin' then
                    return false
                else
                    return uiMgr:getUI(_UIConfig.UILianYin):isClear()
                end
            end
        end
    },
    --===============操作定义
    --打开窗口
    OpenWindow = 
    {
        id = 101,
        name = 'open',
        type = CommandType.Operation,
        callback = function(args)
            uiMgr:openUI(_UIConfig[args[2]], true)
        end
    },
    --关闭窗口
    CloseWindow =
    {
        id = 102,
        name = 'close',
        type = CommandType.Operation,
        callback = function(args)
            uiMgr:closeUIConfig(_UIConfig[args[2]])
        end
    },
    --移动窗口
    MoveTo =
    {
        id = 103,
        name = 'moveto',
        type = CommandType.Operation,
        callback = function(args)
            local window = uiMgr:getUI(_UIConfig[args[2]])
            assert(window, '引导目标窗口' .. args[2] .. '必须打开')
            local path = args[3]
            local x2 = args[4]
            local y2 = args[5]
            local t = args[6] / 1000
            local rt = window:getCom(path, 'RectTransform')
            local x1 = rt.anchoredPosition.x
            local y1 = rt.anchoredPosition.y

            local seq = DG.Tweening.DOTween.Sequence()
            BridgeToLua.TweenTo(seq,0,1,t, function(v)
				rt.anchoredPosition = Vector2.New(
                    x1 + (x2-x1)*v,
                    y1 + (y2-y1)*v
                )end)
        end
    },
    --特殊msg
    Msg = 
    {
        id = 199,
        name = 'msg',
        type = CommandType.Operation,
        callback = function(args)
            local cmd = args[2]

            --放假数据
            if cmd == 'hideLianYinNodeChooseZhaoqin' then
                uiMgr:getUI(_UIConfig.UILianYin):closeWindow_Zhaoqin()
            --隐藏   提亲界面  联姻界面下的 nodeTiqin
            elseif cmd == 'hideLianYinNodeTiqin' then
                uiMgr:getUI(_UIConfig.UILianYin):closeWindow_Tiqin()
            --隐藏  提亲/招亲选项界面  联姻界面下的 nodeChoose
            elseif cmd == 'hideLianYinNodeChoose' then
                uiMgr:getUI(_UIConfig.UILianYin):closeWindow_ZhaoqinTiqin()
            elseif cmd == 'getHongYan' then
                _gameTools.sendMsg(cs_GuideGetWife, {7}, function(data) end)
            end
        end
    },
}

function CommandCompare(a, b, op)

    if op == '=' then 
        return tonumber(a) == tonumber(b)
    elseif op == '>' then
        return tonumber(a) > tonumber(b)
    elseif op == '>=' then
        return tonumber(a) >= tonumber(b)
    elseif op == '<' then
        return tonumber(a) < tonumber(b)
    elseif op == '<=' then
        return tonumber(a) <= tonumber(b)
    elseif op == '!=' then
        return tonumber(a) ~= tonumber(b)
    elseif op == '&&' then
        return tonumber(a)>0 and tonumber(b)>0
    elseif op == '||' then
        return tonumber(a)>0 or tonumber(b)>0
    else
        assert(false, '不支持的操作符, op=' .. tostring(op))
    end
end

--管理器类定义
local M = Class()
CommandData = M

function M:ctor()
    self.commands = {}
end

function M:registerCommand(id, type, name, callback)
    assert(not self.commands[id], '重复注册，条件已经存在，name=' .. tostring(name))
    self.commands[id] = 
    {
        id = id,
        type = type,
        name = name,
        callback = callback
    }
end

function M:nameToID(name)
    for i,v in pairs(self.commands) do
        if v.name == name then
            return v.id
        end
    end
    assert(false, '条件不存在，name=' .. tostring(name))
end

--检测条件
function M:check(id, args)
    local condition = self.commands[id]
    assert(condition and condition.type == CommandType.Condition, '检测失败，命令不是条件或者不存在，id=' .. tostring(id))
    return condition.callback(args)
end

--执行操作
function M:execute(id, args)
    local operation = self.commands[id]
    assert(operation and operation.type == CommandType.Operation, '检测失败，命令不是操作或者不存在，id=' .. tostring(id))
    return operation.callback(args)
end

Command = CommandData.new()

--从配置加入到管理器中去
for i,v in pairs(CommandConfig) do
    Command:registerCommand(v.id, v.type, v.name, v.callback)
end
