--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

gameDataManager = {}

function gameDataManager.init()
    math.randomseed(tostring(os.time()):reverse())  --初始化随机数种子
    gameDataManager.updateHandler = _gameTools.addUpdate(gameDataManager.onUpdate)

    gameDataManager.getLocalData()
    if gameDataManager.playerData == "" or gameDataManager.playerData == nil then
        _gameTools.luaError("本地数据为空,取默认数据")
        gameDataManager.useDefaultData()
    end
    gameDataManager.isLoggedIn = true

    gameDataManager.setFreeGunBtnState(false)
    gameDataManager.clearBag()
    if not gameDataManager.isLoginToday() then
        gameDataManager.loginToday()
    end
end

--region 玩家数据相关接口

gameDataManager.lastUpdateTime = os.time()
gameDataManager.realDeltaTime = 0
gameDataManager.saveDataTimer = 10
gameDataManager.doubleEarning = 1
gameDataManager.gunBagMaxCount = 12 --背包最大数量
gameDataManager.cacheGuideId = 0    --缓存的引导id

function gameDataManager.onUpdate()
    local now = os.time()
    gameDataManager.realDeltaTime = now - gameDataManager.lastUpdateTime
    gameDataManager.lastUpdateTime = now
    --更新数据
    if(gameDataManager.isLoggedIn) then
        --背包数据
        for index in pairs(gameDataManager.bagData) do
            if gameDataManager.bagData[index].level > 0 and gameDataManager.bagData[index].level < 300 then
                gameDataManager.bagData[index].timer = gameDataManager.bagData[index].timer + gameDataManager.realDeltaTime
                if gameDataManager.bagData[index].timer >= gameDataManager.bagData[index].cd then
                    local num = math.floor(gameDataManager.bagData[index].timer / gameDataManager.bagData[index].cd)
                    local moneyNum = gameDataManager.bagData[index].productMoney * gameDataManager.doubleEarning * num
                    gameDataManager.addGold(moneyNum)
                    eventMgr:broadcastEvent(_EventType.ProductMoney, false, { index, moneyNum} )
                    gameDataManager.bagData[index].timer = gameDataManager.bagData[index].timer - gameDataManager.bagData[index].cd * num
                end
            end
        end
    end

    if levelMgr:getCurSceneName() == _ESceneName.MainCity then
        gameDataManager.saveDataTimer = gameDataManager.saveDataTimer - gameDataManager.realDeltaTime
        if gameDataManager.saveDataTimer <= 0 then
            gameDataManager.saveDataTimer = 10
            gameDataManager.saveData()
        end
    end
end

function gameDataManager.clear()
    _gameTools.removeUpdate(gameDataManager.updateHandler)
end


function gameDataManager.useDefaultData()
    gameDataManager.playerData = 
    {
        money = 1000,
        diamond = 0,   
        gameLv = 1,  --玩家当前关卡
        useGunLv = 1,  --玩家当前使用枪的等级
        vigor = 10,   --玩家的精力
        maxGunLv = 1,   --解锁枪的最大等级
        touchMoveSensitivity = 10,  --触屏滑动灵敏度
        expendDiamond = {},  --枪械钻石购买次数
        expendMoney = {},   --抢修金币够没次数
        offlineTime = os.time(),  --离线收益的时间
        signDayNum = 1, --玩家的签到天数
        signTime = 0, --玩家的签到时间
        awardTicket = 999999999,  --玩家转盘抽奖奖券数量
        loginTime = 0,  --玩家登陆时间
        loginDayNum = 0,  --玩家的登陆天数
        finishTaskNum = 0,  --玩家完成的任务数量
        freeGunNum = 0,  --玩家还没领取免费枪支的数量
        vigorTime = os.time(),  --体力恢复时间
        musicState = 1,
        soundState = 1,
        freeUpGunNum = 0,    --免费升级枪支弹窗出现次数
        dayAwardNum = 0,     --每天奖励的次数
        isRefreshFreeBtn = false,    --是否刷新免费按钮的显示
        goldBoxPos = 0,       --黄金宝箱的位置
    }

    gameDataManager.guideData = 
    {
        shootOnHeadTips = 0,
        flashBombTips = 0,
        isShowSettings = 0,
        isShowSnipeGuide = 0,
        finishDataList = {},
    }

    gameDataManager.bagData = { }    --背包数据
    gameDataManager.setBagDataById(1, 1)
end

function gameDataManager.getLocalData() 
    gameDataManager.playerData = _gameTools.strToTable(_gameTools.getLocalData("wreCS_playerData"))
    gameDataManager.bagData = _gameTools.strToTable(_gameTools.getLocalData("wreCS_bagData"))
    gameDataManager.guideData = _gameTools.strToTable(_gameTools.getLocalData("wreCS_guideData"))
end

function gameDataManager.saveData()
    _gameTools.setLocalData("wreCS_playerData", _gameTools.tableToStr(gameDataManager.playerData))
    _gameTools.setLocalData("wreCS_bagData", _gameTools.tableToStr(gameDataManager.bagData))
    _gameTools.setLocalData("wreCS_guideData", _gameTools.tableToStr(gameDataManager.guideData))
end

--获得金币数量
function gameDataManager.getGold()
    return gameDataManager.playerData.money
end

--获得钻石数量
function gameDataManager.getDiamond()
    return gameDataManager.playerData.diamond
end

--获得购买枪的最大等级
function gameDataManager.getMaxGunLv()
    return gameDataManager.playerData.maxGunLv
end

--设置购买枪的最大等级
function gameDataManager.setMaxGunLv(lv)
    gameDataManager.playerData.maxGunLv = lv
end

--获取当前购买枪械的等级
function gameDataManager.getCurGunLv()
    local lv = gameDataManager.playerData.maxGunLv - 4
    return lv <= 1 and 1 or lv
end

--获得玩家当前关卡
function  gameDataManager.getGameLv()    
    if gameDataManager.playerData.gameLv >= _globalCfg.totalGameLv then
        gameDataManager.playerData.gameLv = _globalCfg.totalGameLv
    end
    return gameDataManager.playerData.gameLv
end
function gameDataManager.setGameLv(level)
    if level == nil then
        gameDataManager.playerData.gameLv = gameDataManager.playerData.gameLv + 1
    else
        gameDataManager.playerData.gameLv = level
    end
    if gameDataManager.playerData.gameLv >= _globalCfg.totalGameLv then
        gameDataManager.playerData.gameLv = _globalCfg.totalGameLv
    end
end

--获得玩家当前使用枪的等级
function gameDataManager.getUseGunLv()
    return gameDataManager.playerData.useGunLv
end

--设置玩家当前使用枪的等级
function gameDataManager.setUseGunLv(level)
    gameDataManager.playerData.useGunLv = level
--    gameDataManager.saveData()
end

--获得玩家的剩余精力
function  gameDataManager.getVigor()
    return gameDataManager.playerData.vigor
end

--玩家使用精力
function gameDataManager.useVigor()
    gameDataManager.playerData.vigor = gameDataManager.playerData.vigor - 1
end

--添加精力
function gameDataManager.addVigor(num,state)
    local vigor = gameDataManager.playerData.vigor
    if state then
        vigor = vigor + num
        gameDataManager.playerData.vigor = vigor >= 15 and 15 or vigor
    else
        if vigor < 10 then
            vigor = vigor + num
            gameDataManager.playerData.vigor = vigor >= 10 and 10 or vigor
        end
    end
end

--使用金币
function gameDataManager.useGold(val)
    if val <= gameDataManager.playerData.money then
        gameDataManager.playerData.money = gameDataManager.playerData.money + val
        eventMgr:broadcastEvent(_EventType.RefreshMoney)        
        return true
    else
        return false
    end
end

--使用钻石
function gameDataManager.useDiamond(val)
    if val <= gameDataManager.playerData.diamond then
        gameDataManager.playerData.diamond = gameDataManager.playerData.diamond + val
--        gameDataManager.saveData()
        return true
    else
        return false
    end
end

--添加钻石
function gameDataManager.addDiamond(val)
    gameDataManager.playerData.diamond = gameDataManager.playerData.diamond + val
    eventMgr:broadcastEvent(_EventType.RefreshMoney)
    eventMgr:broadcastEvent(_EventType.RefreshProductPerSec)    
--    gameDataManager.saveData()
end

--添加金币
function gameDataManager.addGold(val)
    gameDataManager.playerData.money = gameDataManager.playerData.money + val
    eventMgr:broadcastEvent(_EventType.RefreshMoney)
    eventMgr:broadcastEvent(_EventType.addFreeGun)
end

--添加枪械金币购买的次数
function gameDataManager.addExpendGoldTimes(pos)
    if not gameDataManager.playerData.expendMoney[pos] then
        gameDataManager.playerData.expendMoney[pos] = 1
    end
    gameDataManager.playerData.expendMoney[pos] = gameDataManager.playerData.expendMoney[pos] + 1
end

--获得某种枪械的gold购买次数
function gameDataManager.getExpendGoldTimes(pos)
    if not gameDataManager.playerData.expendMoney[pos] then
        gameDataManager.playerData.expendMoney[pos] = 1
    end
    return gameDataManager.playerData.expendMoney[pos]
end

--添加枪械钻石购买的次数
function gameDataManager.addExpendDiamondTimes(pos)
    if not gameDataManager.playerData.expendDiamond[pos] then
        gameDataManager.playerData.expendDiamond[pos] = 1
    end
    gameDataManager.playerData.expendDiamond[pos] = gameDataManager.playerData.expendDiamond[pos] + 1
--    gameDataManager.saveData()
end

--获得某种枪械的diamond购买次数
function gameDataManager.getExpendDiamondTimes(pos)
    if not gameDataManager.playerData.expendDiamond[pos] then
        gameDataManager.playerData.expendDiamond[pos] = 1
    end
    return gameDataManager.playerData.expendDiamond[pos]
end

--金币购买消耗计算
function gameDataManager.calExpendGoldNum(pos)
    local dataCfg = dataConfigMgr.getFirearmsByLv(pos)
    local coinPrice = dataCfg.coinPrice
    local mul = dataCfg.coinGrowthValue
    local goldNum = 0
    goldNum = math.modf(coinPrice * (mul ^ (gameDataManager.getExpendGoldTimes(pos) - 1)))
    return goldNum
end

--钻石购买消耗计算
function gameDataManager.calExpendDiamondNum(pos)
    local dataCfg = dataConfigMgr.getFirearmsByLv(pos)
    
    local coinDiamond = dataCfg.diamondPrice
    local conNum = dataCfg.diamondGrowthValue
    local diamondNum = 0
    diamondNum = coinDiamond + (conNum * gameDataManager.getExpendDiamondTimes(pos))
    return diamondNum
end

--背包数据
function gameDataManager.getAllBagData()
    return gameDataManager.bagData
end

--获取单个格子数据
function gameDataManager.getBagDataById(gridId)
    return gameDataManager.bagData[gridId]
end

--设置单个背包格子数据
function gameDataManager.setBagDataById(gridId, level, timer)
    if not gameDataManager.bagData[gridId] then
        gameDataManager.bagData[gridId] = {}
        gameDataManager.bagData[gridId].gridId = gridId
        gameDataManager.bagData[gridId].timer = timer
    end
    gameDataManager.bagData[gridId].timer = timer or 0
    gameDataManager.bagData[gridId].level = level
    if level == 0 or level > 300 then
        gameDataManager.bagData[gridId].cd = 0
        gameDataManager.bagData[gridId].productMoney = 0
    else
        local cfg = dataConfigMgr.getFirearmsByLv(level)
        gameDataManager.bagData[gridId].cd = cfg.productCD
        gameDataManager.bagData[gridId].productMoney = cfg.product
    end

end

--获取每秒产出
function gameDataManager.getProductPerSec()
    local ret = 0
    for index in pairs(gameDataManager.bagData) do
        if gameDataManager.bagData[index].cd ~= 0 then
            ret = ret + gameDataManager.bagData[index].productMoney / gameDataManager.bagData[index].cd
        end
    end
    return ret * gameDataManager.doubleEarning
end

--添加枪械到背包
function gameDataManager.addFirearmsToBag(gunLv)
    for i = 1, gameDataManager.gunBagMaxCount do
        local data = gameDataManager.getBagDataById(i)
        if data == nil or data.level == 0 then
            gameDataManager.setBagDataById(i, gunLv)
            eventMgr:broadcastEvent(_EventType.RefreshProductPerSec)   
            eventMgr:broadcastEvent(_EventType.addFreeGun)         
            return i
        end
    end
end

--添加枪械到背包某一个格子
function gameDataManager.addFirearmsToGrid(pos,gunLv)
    gameDataManager.setBagDataById(pos, gunLv)
    eventMgr:broadcastEvent(_EventType.RefreshProductPerSec)   
    eventMgr:broadcastEvent(_EventType.addFreeGun)         
end

--判断背包是否已满
function gameDataManager.bagIsFull()
    local res = nil
    for i = 1, gameDataManager.gunBagMaxCount do
        local data = gameDataManager.getBagDataById(i)
        if data == nil or data.level == 0 then
            res = i
            break
        end
    end
    return res
end

--背包清理
function gameDataManager.clearBag()
    for i = 1, gameDataManager.gunBagMaxCount do
        local data = gameDataManager.getBagDataById(i)
        if data and data.level > 300 then
            data.level = 0
        end
    end
    gameDataManager.setGoldBoxPos(0)
end

--计算枪的伤害属性
function gameDataManager.calDamageRatio(lv)
    local val = (dataConfigMgr.getFirearms(lv).atk / math.ceil(lv / 5)) / 200
    return val
end

--计算枪的后坐力属性
function gameDataManager.calRecoil(lv)
    local val = (2 - dataConfigMgr.getFirearms(lv).force) / 2
    return val
end

--计算枪的弹夹容量属性
function gameDataManager.calMagazieSize(lv)
    local val = dataConfigMgr.getFirearms(lv).clipSize / 60
    return val
end

--计算枪的射速属性
function gameDataManager.calAtkSpeed(lv)
    local val = dataConfigMgr.getFirearms(lv).speed / 700
    return val
end

--计算枪的稳定性属性
function gameDataManager.calStabilize(lv)
    local val = dataConfigMgr.getFirearms(lv).stability
    return val
end

--游戏胜利获得金币数量
function gameDataManager.victoryGetGold()
    local victoryCfg = _globalCfg.resultProductNum[1]
    local lv = gameDataManager.getCurGunLv()
    local gold = gameDataManager.calExpendGoldNum(lv) * ( math.random() * (victoryCfg[2] - victoryCfg[1]) + victoryCfg[1])
    return math.floor(gold)
end

--游戏失败获得金币数量
function gameDataManager.defaultGetGold()
    local defaultCfg = _globalCfg.resultProductNum[2]
    local lv = gameDataManager.getCurGunLv()
    local gold = gameDataManager.calExpendGoldNum(lv) * ( math.random() * (defaultCfg[2] - defaultCfg[1]) + defaultCfg[1])
    return math.floor(gold)
end

--计算离线收益获得金币数量
function gameDataManager.calOfflineGold(time)
    local realTime = time
    local maxTime = _globalCfg.offlineMaxTime * 3600
    local perGold = gameDataManager.getProductPerSec()
    local goldNum = 0
    if perGold <= 0 then
        goldNum = 100
    else
        if realTime >= maxTime then
            goldNum = maxTime * perGold
        else
            goldNum = realTime * perGold
        end
    end
    return goldNum
end

--获得音乐的状态
function gameDataManager.getMusicState()
    return gameDataManager.playerData.musicState == 1
end

--设置音乐的状态
function gameDataManager.setMusicState(state)
    gameDataManager.playerData.musicState = state and 1 or 0
end

--获得音效的状态
function gameDataManager.getSoundState()
    return gameDataManager.playerData.soundState == 1
end

--设置音效的状态
function gameDataManager.setSoundState(state)
    gameDataManager.playerData.soundState = state and 1 or 0
end

--获得滑动速度
function gameDataManager.getSlideSpeed()
    return gameDataManager.playerData.touchMoveSensitivity
end

--设置滑动速度
function gameDataManager.setSlideSpeed(speed)
    gameDataManager.playerData.touchMoveSensitivity = speed
end

--计算免费领取金币数量
function gameDataManager.calFreeGold()
    return math.floor(gameDataManager.calExpendGoldNum(gameDataManager.getCurGunLv()) * _globalCfg.freeGoldNum)
end

--计算免费领取钻石数量
function gameDataManager.calFreeDiamond()
    local gunLv = gameDataManager.getMaxGunLv() - 2 > 0 and gameDataManager.getMaxGunLv() - 2 or 1
    return math.ceil((120 - gunLv) / 120 * _globalCfg.freeDiamondNum * gameDataManager.calExpendDiamondNum(gunLv))
end

--今日首次登录
function gameDataManager.loginToday()
    gameDataManager.addLoginDay()

    gameDataManager.playerData.loginTime = os.time()
    gameDataManager.playerData.freeUpGunNum = 0
    gameDataManager.playerData.dayAwardNum = 0

    if gameDataManager.playerData.signDayNum >= 8 then    
        gameDataManager.playerData.signDayNum = 1
    end
end

--增加登陆天数
function gameDataManager.addLoginDay()
    if not gameDataManager.playerData.loginDayNum then
        gameDataManager.playerData.loginDayNum = 0
    end
    gameDataManager.playerData.loginDayNum = gameDataManager.playerData.loginDayNum + 1
end

--获得玩家的登陆天数
function gameDataManager.getLoginDay()
    if not gameDataManager.playerData.loginDayNum then
        gameDataManager.playerData.loginDayNum = 0
    end
    return gameDataManager.playerData.loginDayNum
end

--增加签到天数
function gameDataManager.addSignDay()
    gameDataManager.playerData.signDayNum = gameDataManager.playerData.signDayNum + 1
end
    
--获得签到天数
function gameDataManager.getSignDay()
    if not gameDataManager.playerData.signDayNum then
        gameDataManager.playerData.signDayNum = 1
    end
    return gameDataManager.playerData.signDayNum
end

--获得玩家签到时间
function gameDataManager.getSignTime()
    if not gameDataManager.playerData.signTime then
        gameDataManager.playerData.signTime = 0
    end
    return gameDataManager.playerData.signTime
end

--设置玩家签到时间
function gameDataManager.setSignTime()
    gameDataManager.playerData.signTime = os.time()
end

--是否已经签到
function gameDataManager.isSignToday()
    local todayDate = os.time()
    local sysDate = gameDataManager.getSignTime()
    local dl = math.floor(sysDate / 24 / 60 / 60)
    local dn = math.floor(todayDate / 24 / 60 / 60)
    if dl ~= dn then
        return false
    else
        return true
    end
end

--获得玩家的登陆时间
function gameDataManager.getLoginTime()
    if not gameDataManager.playerData.loginTime then
        gameDataManager.playerData.loginTime = 0
    end
    return gameDataManager.playerData.loginTime
end

--今日是否登录过
function gameDataManager.isLoginToday()
    local todayDate = os.time()
    local sysDate = gameDataManager.getLoginTime()
    local dl = math.floor(sysDate / 24 / 60 / 60)
    local dn = math.floor(todayDate / 24 / 60 / 60)
    return dl == dn
end

--获得玩家奖券的数量
function gameDataManager.getTicket()
    if not gameDataManager.playerData.awardTicket then
        gameDataManager.playerData.awardTicket = 4
    end
    return gameDataManager.playerData.awardTicket
end

--添加奖券的数量
function gameDataManager.addTicket(num)
    local ticket = gameDataManager.playerData.awardTicket
    gameDataManager.playerData.awardTicket = ticket + num >= 4 and 4 or ticket + num
end

--使用奖券
function gameDataManager.useTicket()
    local num = gameDataManager.playerData.awardTicket - 1
    if num < 0 then
        return false
    else
        gameDataManager.playerData.awardTicket = num
        return true
    end
end

--获得离线收益的时间
function gameDataManager.getOfflineTime()
    return gameDataManager.playerData.offlineTime
end

--设置离线收益的时间
function gameDataManager.setOfflineTime()
    gameDataManager.playerData.offlineTime = os.time()
end

--是否有离线收益
function gameDataManager.isOfflineIncome()
    if os.time() - gameDataManager.getOfflineTime() > 3 * 60  then        
        return os.time() - gameDataManager.getOfflineTime()
    else
        return false
    end
end

--获得玩家完成的任务数量
function gameDataManager.getFinishTaskNum()
    if not gameDataManager.playerData.finishTaskNum then
       gameDataManager.playerData.finishTaskNum = 0 
    end
    return gameDataManager.playerData.finishTaskNum
end

--添加玩家完成的任务数量
function gameDataManager.addFinishTaskNum()
    gameDataManager.playerData.finishTaskNum = gameDataManager.playerData.finishTaskNum + 1
end

--是否完成新手引导
function gameDataManager.isFinishGuide(id)
    if not gameDef.GuideEnable then
        return false
    end
    for i = 1, #gameDataManager.guideData.finishDataList do
        if gameDataManager.guideData.finishDataList[i] == id then
            return true
        end
    end
    return false
end

--完成某步新手引导
function gameDataManager.finishGuide(id)
    table.insert(gameDataManager.guideData.finishDataList, id)
end

--设置收益为双倍或单倍
function gameDataManager.setEarningDouble(args)
    gameDataManager.doubleEarning = args
    eventMgr:broadcastEvent(_EventType.RefreshProductPerSec) 
end

--获得还没领取免费枪支的数量
function gameDataManager.getFreeGunNum()
    if not gameDataManager.playerData.freeGunNum then
        gameDataManager.playerData.freeGunNum = 0
    end
    return gameDataManager.playerData.freeGunNum
end

--使用还没领取免费枪支的数量
function gameDataManager.useFreeGunNum()
    local num = gameDataManager.getFreeGunNum()
    if num > 0 then
        gameDataManager.playerData.freeGunNum = num - 1
    end
end

--设置还没领取免费枪支的数量
function gameDataManager.setFreeGunNum(num)
    gameDataManager.playerData.freeGunNum = num
end

--随机生成一支免费枪
function gameDataManager.randomGetGun()
    return math.floor( math.random() * gameDataManager.getCurGunLv() + 1)
end

--背包中是否有空格
function gameDataManager.isEmptyPosInBag()
    local res = false
    for i = 1, 12 do
        local data = gameDataManager.getBagDataById(i)
        if data == nil or data.level == 0 then
            res = true
        end
    end
    return res
end

--获得体力恢复时间
function gameDataManager.getFreeVigorTime()
    if not gameDataManager.playerData.vigorTime then
        gameDataManager.playerData.vigorTime = os.time()
    end
    return gameDataManager.playerData.vigorTime
end

--设置体力恢复时间
function gameDataManager.setFreeVigorTime()
    gameDataManager.playerData.vigorTime = os.time() + 60 * 10 
end

--获得免费升级弹窗次数
function gameDataManager.getFreeUpGunNum()
    if not gameDataManager.playerData.freeUpGunNum then
        gameDataManager.playerData.freeUpGunNum = 0
    end
    return gameDataManager.playerData.freeUpGunNum
end

--添加免费升级弹窗次数
function gameDataManager.addFreeUpGunNum()
    gameDataManager.playerData.freeUpGunNum = gameDataManager.playerData.freeUpGunNum + 1
end

--获得每天奖励的次数
function gameDataManager.getDayAwardNum()
    if not gameDataManager.playerData.dayAwardNum then
        gameDataManager.playerData.dayAwardNum = 0
    end
    return gameDataManager.playerData.dayAwardNum
end

--添加每天奖励的次数
function gameDataManager.addDayAwardNum()
    gameDataManager.playerData.dayAwardNum = gameDataManager.playerData.dayAwardNum + 1
end

--获得主界面是否显示免费枪支按钮状态
function gameDataManager.getFreeGunBtnState()
    if not gameDataManager.playerData.isRefreshFreeBtn then
        gameDataManager.playerData.isRefreshFreeBtn = false
    end
    return gameDataManager.playerData.isRefreshFreeBtn
end

--设置主界面是否显示免费枪支按钮状态
function gameDataManager.setFreeGunBtnState(state)
    gameDataManager.playerData.isRefreshFreeBtn = state
end

--是否显示签到关卡
function gameDataManager.isShowSignPanel()
    if gameDataManager.getGameLv() >= _globalCfg.showSignLv then
        return true
    else
        return false   
    end
end

--获得黄金宝箱的位置
function gameDataManager.getGoldBoxPos()
    if not gameDataManager.playerData.goldBoxPos then
        gameDataManager.playerData.goldBoxPos = 0
    end
    return gameDataManager.playerData.goldBoxPos
end

--设置黄金宝箱的位置
function gameDataManager.setGoldBoxPos(pos)
    gameDataManager.playerData.goldBoxPos = pos
end

--endregion
