--[[
    物品数据管理

    items   全部物品，以sid作为索引,无法排序，也通过getList返回标准table，进行排序
    items包含如下三页可排序的标准数组数据
        items_use 第一页，(class=1)用于直接或者间接使用的普通物品
            属性:count
        items_compose 第二页，(class=2)用于合成的特殊物品
            属性:无
        items_wear 第三页，(class=3)用于佩戴的时装物品
            属性:

    小分类：
            EXP = 1,// 政绩经验
			MONEY = 2,// 金币
			GOLD = 3,// 元宝
			FOOD = 4,// 粮草
			SOLDIER = 5,// 士兵
			WIFE_ENERGY = 6,// 红颜宠幸精力
			BUSINESS_COUNT = 7,// 商业次数
			AGRICULTURE_COUNT = 8,// 农业次数
			RECRUIT_COUNT = 9,// 士兵次数
			ITEM = 10,// 道具
			YAMEN_CURRENCY = 11,// 衙门货币
			VIP_EXP = 12,// vip经验
			PRESTIGE_MAX = 13,// 声望上限奖励值
			PRESTIGE = 14,// 声望奖励值
			WIFE = 15,// 红颜
			MENKE_WULI = 16,// 门客武力
			MENKE_ZHILI = 17,// 门客智力
			MENKE_ZHENGZHI = 18,// 门客政治
			MENKE_MEILI = 19,// 门客魅力
			MENKE_BOOK_EXP = 20,// 门客书籍经验
			MENKE_SKILL_EXP = 21,// 门客技能经验
			WIFE_SKILL_EXP = 22,// 红颜技能经验
			WIFE_MEILI = 23,// 红颜魅力
			WIFE_INTIMACY = 24,// 红颜亲密度
			XUNFANG_ENERGY = 25,// 寻访精力
			CHILDREN_ENERGY = 26,// 子嗣活力
			ITEM_GIFT = 27,// 礼包
			ITEM_TITLE_FASHION = 28,// 称号时装
			ITEM_PLAYER_FASHION = 29,// 玩家时装
			ITEM_NPC_FASHION = 30,// NPC时装
			ITEM_MENKE_RANDOM = 31,// 门客随机属性
    大分类：
        USE = 1
            子类：剩余其他的
        COMPOSE = 2
            子类：syntheticMaterial不为空
        WEAR = 3
            子类：28,29,30
            

    道具item的属性
    sid SID
    count 当前数量
    using 是否使用中
    endtime 使用结果时间
    overtime  有效期
    sortid 排序码
    quality 品质
    red 是否有小红点
    name 名称
    desc 描述
    icon_atlas 图标图集
    icon 图标
    border_atlas 边框图集
    border 边框图标
    composeItem 合成消耗数组
    type 小类型
    useForMenke 门客属性加成类型
    useForWear 穿戴类型
    usable 可使用
]]

local M = {}
gameDataManager.itemData = M

M.maxSize = 300 --道具最大数量
M.anchorNode = nil --获得道具的锚点坐标
M.loaded = false

--请求数据
function M:setItemData()
	_gameTools.sendMsg(cs_GET_ITEMS, {1}, self.loadAll, self)
end

--加载所有
function M:loadAll(data)
    --print('玩家物品列表')
    --for i,v in ipairs(data.items) do
    --    print(v.sid, v.count, v.startTime, v.use)
    --end

    --缓存起来
    self.itemBorders = splitToString(t_global[16].value)
    --原始模版
    self:clear()
    for i,v in pairs(t_item) do
        if t_item[v.sid] then
            self:load(v.sid)
        end
    end
    self.itemBorders = nil

    --读取服务器拿到的道具数量
    for i,v in ipairs(data.items) do
        if t_item[v.sid] then
            local item = self:get(v.sid)
            if item then
                item.count = v.count
                item.using = v.use == 1
                item.endtime = tonumber(tostring(v.overTime))
            end
        end
    end

    local sortFunc = function(a,b)
        return a.sortid<b.sortid
    end
    table.sort(self.items_use, sortFunc)
    table.sort(self.items_compose, sortFunc)
    table.sort(self.items_wear, sortFunc)

    self:refreshRed()

    --立即刷新当前用户的时装
    self:resetFashion(gameDataManager.playerInfo)

    self.loaded = true
    gameDataManager.menkeData:refreshRed()
end

--应用玩家时装信息到道具里面去
function M:resetFashion(data)
    self.fashionData = data

    --先刷新时间,data为空则直接从道具表刷新可见时装
    if data then
        if #data.palaceList > 0 then
            for i=1,#data.palaceList/2,1 do
                local sid = tonumber(tostring(data.palaceList[(i-1)*2+1]))
                local endtime = tonumber(tostring((data.palaceList[(i-1)*2+2])))
                self:get(sid).endtime = endtime
            end
        end
        if #data.clothesList > 0 then
            for i=1,#data.clothesList/2,1 do
                local sid = tonumber(tostring((data.clothesList[(i-1)*2+1])))
                local endtime = tonumber(tostring((data.clothesList[(i-1)*2+2])))
                self:get(sid).endtime = endtime
            end
        end
        if #data.npcList > 0 then
            for i=1,#data.npcList/2,1 do
                local sid = tonumber(tostring((data.npcList[(i-1)*2+1])))
                local endtime = tonumber(tostring((data.npcList[(i-1)*2+2])))
                self:get(sid).endtime = endtime
            end
        end
    end
end

function M:getOtherFashionData(level, titles, fashionItemID)
    return 
    {
        titles = splitToLanguage(titles),
        image = fashionItemID > 0 and self:get(fashionItemID).fashionImage or t_government[level].officerImg,
    }
end

--[[
    计算玩家自己的时装信息
    返回值
    {
        titles 称号列表   
        currentTitle 当前称号
        image 时装图片 
    }
--]]
function M:getFashionData(level)
    local data = self.fashionData

    local ret = {}
    ret.currentTitle = nil
    ret.image = nil
    ret.titles = {}
    ret.titleItems = {}

    if data then
        if #data.palaceList > 0 then
            for i=1,#data.palaceList/2,1 do
                local sid = tonumber(tostring((data.palaceList[(i-1)*2+1])))
                local item = self:get(sid)
                table.insert(ret.titles, item.fashionTitle)
                table.insert(ret.titleItems, sid)
                ret.image = item.fashionImage
                ret.currentTitle = item.fashionTitle
            end
        end
        if #data.clothesList > 0 then
            for i=1,#data.clothesList/2,1 do
                local sid = tonumber(tostring((data.clothesList[(i-1)*2+1])))
                local item = self:get(sid)
                if not ret.image then
                    ret.image = item.fashionImage
                end
            end
        end
    end

    --没有时装再走官品服饰
    if not ret.image then
        ret.image = t_government[level].officerImg
    end
    
    return ret
end

--清空物品列表
function M:clear()
    self.items = {}
    self.items_use = {}
    self.items_compose = {}
    self.items_wear = {}
end

--加载物品数据
function M:load(sid)
    local data = t_item[sid]
    assert(data, "加载物品失败，物品不存在，sid=" .. sid)
    local item = {}
    item.sid = sid
    --属性，当前数量
    item.count = 0
    item.visible = data.visible
    --属性，使用状态，大类3使用
    item.using = false --是否使用中
    item.endtime = 0 --结束时间
    item.sortid = data.sort
    item.quality = data.quality
    item.red = false
    item.name = dataConfigMgr.getLanguage(data.nameid)
    item.desc = dataConfigMgr.getLanguage(data.descid)
    --item.icon_atlas = data.icon_atlas
    --道具有默认的图集，不需要配置
    item.icon_atlas = data.icon_atlas
    item.icon = data.icon
    item.border_atlas = self.itemBorders[1]
    item.border = self.itemBorders[1+item.quality]
    item.fashionTitle = dataConfigMgr.getLanguage(data.fashionTitleID)
    item.fashionImage = data.fashionImage
    item.overtime = data.fashionOverTime --有效期
    item.composeItem = {}
    if #data.syntheticMaterial > 0 then
        for i=1,(#data.syntheticMaterial)/2,1 do
            table.insert(item.composeItem,
                { 
                    sid = data.syntheticMaterial[(i-1)*2+1],
                    count = data.syntheticMaterial[(i-1)*2+2]
                })
        end
    end
    item.type = data.type
    --用途分类
    item.useForMenke = 
        item.type==31 or 
        (item.type>=16 and item.type<=19) or
        item.type==20 or
        item.type==21
    item.useForWear =
        item.type==28 or 
        item.type==29 or 
        item.type==30
    item.useForCompose = 
        #item.composeItem > 0
    item.usable = 
        item.useForWear or 
        item.useForMenke or 
        item.type==27 or 
        item.type==22 or
        data.use==1


    if item.visible==1 then
        if item.useForWear then
            table.insert(self.items_wear, item)
        else
            table.insert(self.items_use, item)
        end

        if item.useForCompose then
            table.insert(self.items_compose, item)
        end
    end

    self.items[item.sid] = item
end

--[[
    使用物品或者增加物品
    count 使用数量
        count > 0 表示正常使用，不足时会弹出道具并且不会进入回调
        count < 0 表示增加（增加道具不弹提示，要提示请调用get）
        count = 0表示不做改变，但是会判断数量，道具不足时不会进入回调
]]
function M:use(sid, count, callback)
    local item = self.items[sid]

    assert(item, "物品不存在，sid=" .. sid)

    if not count then
        count = 1
    end

    if count == 0 and item.count==0 then
        return false
    end

    if count > 0 and item.count-count<0 then
        alert('道具不足')
        return false
    end

    item.count = item.count - count

    if callback then
        callback()
    end
    
    return true;
end

--[[
    获得物品或者减少物品
    count 增加数量
]]
function M:add(sid, count, callback)
    local item = self.items[sid]

    assert(item, "物品不存在，sid=" .. sid)

    if not count then
        count = 1
    end

    if item.count + count < 0 then
        assert(false, '道具数量不能为负')
    end

    item.count = item.count + count

    if callback then
        callback()
    end
    
    return true;
end

--检查使用道具的剩余空间
function M:checkSize(size, callback)
    local c = 0
    for i,v in pairs(self.items) do
        if v.class== 1 and v.count>0 then
            c = c+1
        end
    end
    if c >= size then
        if callback then
            callback()
        end
        return true
    else
        return false
    end
end

--返回单个对象
function M:get(sid)
    local item = self.items[sid]
    assert(item, '道具不存在，id=' .. tostring(sid))
    return item
end

--返回某个大类的有序数组
function M:getList(class)
    if class==1 then
        local result = {}
        for i,v in pairs(self.items_use) do
            if v.count > 0 then
                table.insert(result, v)
            end
        end
        return result
    elseif class==2 then
        return self.items_compose
    elseif class==3 then
        return self.items_wear
    end  
end

--刷新小红点
function M:refreshRed()
    self.red = false
    for i,v in pairs(self.items) do
        if #v.composeItem > 0 then
            v.red = true
            for j,w in pairs(v.composeItem) do
                local it = self:get(w.sid)
                if it.count < w.count then
                    v.red = false
                end
            end
            self.red = self.red or v.red
        else
            v.red = false --合成小红点
        end
    end
    return false,self.red,false
end