
--[[
	门客结构
	sid = SID
	name 名称
	level 等级
	exp = 经验（消耗银两）
	exp_book 书籍经验
	exp_skill 技能经验
	title 爵位等级
	specialsID 特长ID数组
	specialsName （UI）特长
	icon_atlas 图集
	icon = 图标
	bsxImg 半身像
	books 门客拥有的资质书籍
	skills 门客拥有的技能
	bornSkill 出生自带书籍
	bornSkillInit 出生自导资质（暂时不用）
	ALL = 属性总加成
	WL = --武力总加成值
	ZL = --武力总加成值
	ZZ = --武力总加成值
	ML = --武力总加成值
	maxlevel 当前爵位最大等级
	maxlevel_book 当前爵位最大书籍等级
	maxlevel_skill 当前爵位最大技能等级
	costMoney 升级消耗
	titleName (UI)爵位名称
	titleNameLast （UI）上级爵位名称
	titleValueLast （UI）上级爵位等级
	titleNameCurrent （UI）当前爵位名称
	titleValueCurrent （UI）当前爵位等级
	titleNameNext（UI）下级爵位名称
	titleValueNext（UI）下级爵位名称
	titleIcon_atlas 爵位图集
	titleIcon 爵位图标
	titleBg_atlas 爵位背景图集
	titleBg 爵位北京图片
]]
Menke = Class()

--[[
	技能结构

	menke 所属门客对象引用
	sid SID
	level 技能等级
	name 技能名称
	type 技能类型
	icon_atlas 图集
	icon 图标	
	
	red 是否显示小红点
	costExp 升级经验
	text (UI)名称
	currentValue (UI)当前效果
	nextValue (UI)下级效果
	costValue (UI)升级经验
]]
Skill = Class()

--[[
	书籍结构

	menke 所属门客对象引用
	sid SID
	level 等级
	name 名称
	type 类型
	typeName (UI) 类型
	icon_atlas 图集
	icon 图标
	star 星级
	value 资质值
	red 是否显示小红点

	costExp 升级经验
	text (UI)名称
	currentValue (UI)当前资质
	nextValue (UI)下级资质
	costValue (UI)升级消耗
]]
Book = Class()

local M = {}
gameDataManager.menkeData = M
M.menkes = {}
M.red = false

function M:setMenkeList(list)
	self.menkes = {}
	for i,v in ipairs(list) do
		local menke = Menke.new(v)
		menke:refresh()
		self.menkes[v.sid] = menke
	end

	self:refreshRed()
end

function M:refreshRed()
	self.red = false
	for i,v in pairs(self.menkes) do
		v:refreshRed()
		self.red = self.red or v.red
	end
end

function M:addMenke(data)
	local menke = Menke.new(data)
	menke:refresh()
	self.menkes[data.sid] = menke
	gameDataManager.menkeData:refreshRed()
end

--返回有序的门客列表
function M:getMenkeList(filter, sorter)
	local result = {}
	for i,v in pairs(self.menkes) do
		if not filter or filter(v)==true then
			table.insert(result, v)
		end
	end
	if sorter then
		table.sort(result, sorter)
	end
	return result
end

--返回门客对象
function M:get(sid)
	local menke = self.menkes[sid]
	assert(menke, '门客不存在，sid=' .. tostring(sid))
	return menke
end

function Menke:ctor(info)
	local data = t_menke[info.sid]
	local specialsEnum = splitToLanguage(t_global[2].value) --特长名称枚举
	assert(data, '门客不存在,sid=' .. tostring(info.sid))
	self.sid = info.sid
	self.name =dataConfigMgr.getLanguage(data.nameid)
	self.level = info.level;
	self.exp = tonumber(tostring(info.moneyexp));
	self.exp_book = info.bookexp;
	self.exp_skill = info.skillexp;
	self.title = info.title;--爵位
	self.icon_atlas = data.head_atlas
	self.icon = data.head
    self.bsxImg = data.bsxImg
	self.books = {}
	self.skills = {}
	self.bornSkill = {}
	self.bornSkillInit = {}
	self.WL = info.fight
	self.ZL = info.intelligence
	self.ZZ = info.politics
	self.ML = info.charm
	self.sounds = data.sounds
	local bornSkillDatas = splitToString(data.skill, ';')
	for i,v in ipairs(bornSkillDatas) do
		local bornSkillData = splitToNumber(v, ',')
		self.bornSkillInit[bornSkillData[1]] = bornSkillData[2]
		self.bornSkill[bornSkillData[1]] = bornSkillData[3]
	end
	self.specialsID = splitToNumber(data.special, ',')
	self.specialsName = {}
	for i,v in ipairs(self.specialsID) do
		table.insert(self.specialsName, specialsEnum[v])
	end
	self.specialsName = contactToString(self.specialsName)
	for i,v in ipairs(info.aptitudeList) do
		local book = Book.new(self,v.aptitudeid, v.level)
		table.insert(self.books, book)
	end
	for i,v in ipairs(info.skillList) do
		local skill = Skill.new(self,v.skillid,v.level)
		table.insert(self.skills, skill)
	end
end

--计算门客某个类型的资质总合
function Menke:getZZ(type)
	local value = 0
	for i,v in pairs(self.books) do
		if v.type == type then
			value = value + v.star * v.level
		end
	end
	return value
end

--获取门客战斗力
function Menke:getFightPower()
    --公式：等级*武力资质*5000+武力值
    local fightPower = self:getZZ(1) * self.level * 5000 + self.WL
    return fightPower
end

function Menke:refresh()
	self.titleName = dataConfigMgr.getLanguage(t_menkeTitle[self.title].nameid)
	self.maxlevel = t_menkeTitle[self.title].maxlevel_menke
	self.maxlevel_book = t_menkeTitle[self.title].maxlevel_book
	self.maxlevel_skill = t_menkeTitle[self.title].maxlevel_skill
	self.costMoney = 0.5*math.pow(self.level,3) + 2*math.pow(self.level,2) + 49.5*self.level + 48
	self.hasLast = t_menkeTitle[self.title-1] ~= nil
	self.hasNext = t_menkeTitle[self.title+1] ~= nil
	
	self.titleNameLast = self.hasLast and dataConfigMgr.getLanguage(t_menkeTitle[self.title - 1].nameid) or '' --一般不出出现空字串
	self.titleValueLast = self.hasLast and t_menkeTitle[self.title-1].maxlevel_menke or '' --一般不出出现空字串
	self.titleNameCurrent = dataConfigMgr.getLanguage(t_menkeTitle[self.title].nameid)
	self.titleValueCurrent = t_menkeTitle[self.title].maxlevel_menke
	self.titleNameNext = self.hasNext and dataConfigMgr.getLanguage(t_menkeTitle[self.title+1].nameid) or dataConfigMgr.getLanguage(4910)
	self.titleValueNext = self.hasNext and t_menkeTitle[self.title+1].maxlevel_menke or dataConfigMgr.getLanguage(4911)
	
	self.titleIcon_atlas = t_menkeTitle[self.title].icon_atlas
	self.titleIcon = t_menkeTitle[self.title].icon
	self.titleBg_atlas = t_menkeTitle[self.title].image_atlas
	self.titleBg = t_menkeTitle[self.title].image
	self.titleReward = t_menkeTitle[self.title].award_bookexp

	self.headBg = t_menkeTitle[self.title].head_bg_image
	self.headBorder = t_menkeTitle[self.title].head_frame_image

	self.costItems = {}
	local costItemsStr = splitToString(t_menkeTitle[self.title].cost, ';')
	for i,v in pairs(costItemsStr) do
		local item = splitToNumber(v)
		local costItem = {}
		costItem.sid = item[1]
		costItem.cost = item[2]
		table.insert(self.costItems, costItem)
	end

	for i,v in pairs(self.books) do
		v:refresh()
	end
	
	for i,v in pairs(self.skills) do
		v:refresh()
	end

	--总资质
	self.ALL = self.WL+self.ZL+self.ZZ+self.ML

	--模拟初始资质
	self.sortZZ = 0
	for i,v in pairs(self.books) do
		self.sortZZ = self.sortZZ + v.star * 1
	end

	--战斗力刷新
	self.fightPower = self:getFightPower()

	self:refreshRed()
end

function Menke:refreshRed()
	self.red = false
	self.red_book = false
	self.red_skill = false
	
	for i,v in pairs(self.books) do
		v:refreshRed()
		self.red_book = self.red_book or v.red
	end
	
	for i,v in pairs(self.skills) do
		v:refreshRed()
		self.red_skill = self.red_skill or v.red
	end

	self.red = self.red_book or self.red_skill

	--门客本身的提拔小红点
	local isMaxLevel = self.level >= self.maxlevel
	local isMaxTitle = t_menkeTitle[self.title].nexttitle == 0
	local canPromote = (isMaxLevel and not isMaxTitle)
	self.red = self.red or canPromote
	--可提拔且道具满足
	if gameDataManager.itemData.loaded and canPromote then
		local isEnough = true
		for i=1,3 do
			local cost = self.costItems[i]
			if cost then
				local item = gameDataManager.itemData:get(cost.sid)
				if item.count < cost.cost then
					isEnough = false
				end
			end
		end
		self.red_promote = isEnough
		self.red = self.red and self.red_promote
	else
		self.red_promote = false
	end
end

function Menke:getSpecialExist(specialID)
	return 
	getValueExist(self.specialsID, specialID) or 
	getValueExist(self.specialsID, 5) or 
	getValueExist(self.specialsID, 6)
end

function Book:ctor(menke, sid, level)
	local data = t_book[sid]
	local bookTypeNameEnum = splitToLanguage(t_global[3].value)
	local bookImagePathEnum = splitToString(t_global[4].value)
	assert(data, '书籍不存在，sid=' .. tostring(sid))
	self.menke = menke
	self.sid = sid
	self.level = level
	self.name =dataConfigMgr.getLanguage(data.nameid)
	self.type = data.type
	self.typeName = bookTypeNameEnum[data.type]
	self.icon_atlas = bookImagePathEnum[1]
	self.icon = bookImagePathEnum[data.type + 1]
	self.star = data.star
	self.costExp = t_book[self.sid].exp
	self.bookid = data.bookid
	self.bookodds = data.bookodds
end

function Book:refresh()
	self.value = self.star * self.level
	self.text = '《' .. self.name .. '》' .. dataConfigMgr.getLanguage(10001) .. self.level
	self.currentValue = string.format(dataConfigMgr.getLanguage(4908), self.typeName, self.star * self.level)
	if self.level >= self.menke.maxlevel_book then
		self.nextValue = dataConfigMgr.getLanguage(4907)
		self.costValue = dataConfigMgr.getLanguage(4907)
		self.red = false
		self.isMax = true

		if t_menkeTitle[self.menke.title].nexttitle <=0 then
			self.nextValue = dataConfigMgr.getLanguage(4927)
			self.isFull = true
		else
			self.isFull = false
		end
	else
		self.nextValue = string.format(dataConfigMgr.getLanguage(4909), self.typeName, self.star * (self.level+1))
		self.costValue = self.costExp
		self.isMax = false
		self.isFull = false

		
		local item = (gameDataManager.itemData and gameDataManager.itemData.loaded) and gameDataManager.itemData.items[5400 + self.type] or nil
		local itemEnough = item and item.count > 0 or false
		self.red = (self.menke.exp_book >= self.costExp or itemEnough) and (not isMax) --todo检测道具
	end
end

function Book:refreshRed()
	if self.level >= self.menke.maxlevel_book then
		self.red = false
	else
		local item = (gameDataManager.itemData and gameDataManager.itemData.loaded) and gameDataManager.itemData.items[5400 + self.type] or nil
		local itemEnough = (item~= nil and item.count > 0)
		--门客道具不走红点
		itemEnough = false
		self.red = (self.menke.exp_book >= self.costExp or itemEnough) and (not isMax) --todo检测道具
	end
end

function Skill:ctor(menke, sid, level)
	local data = t_skill[sid]
	assert(data, '技能不存在,sid=' .. tostring(sid))
	
	self.menke = menke
	self.sid = sid
	self.level = level
	
	self.name =dataConfigMgr.getLanguage(data.nameid)
	self.type = data.skillType
	self.icon_atlas = data.icon_atlas
	self.icon = data.icon
end

function Skill:refresh()
	local skillTypeNameEnum = splitToLanguage(t_global[1].value)
	self.text = self.name .. dataConfigMgr.getLanguage(10001) .. self.level
	self.costExp = t_skillExp[self.level].exp_skill
	self.currentValue = self:calcValue(self.level) .. '%' .. skillTypeNameEnum[self.type]
	if self.level >= self.menke.maxlevel_skill then
		self.nextValue = dataConfigMgr.getLanguage(4903)
		self.costValue = dataConfigMgr.getLanguage(4903)
		self.red = false
		self.isMax = true

		if t_menkeTitle[self.menke.title].nexttitle <=0 then
			self.isFull = true
		else
			self.isFull = false
		end
	else
		self.isMax = false
		self.isFull = false

		self.nextValue = self:calcValue(self.level + 1) .. '%' .. skillTypeNameEnum[self.type]
		self.costValue = string.format(dataConfigMgr.getLanguage(4902), self.costExp)
		self.red = self.menke.exp_skill >= self.costExp
	end
end

function Skill:refreshRed()
	if self.level >= self.menke.maxlevel_skill then
		self.red = false
	else
		self.red = self.menke.exp_skill >= self.costExp
	end
end

function Skill:calcValue(level)
	return (self.menke.bornSkillInit[self.sid] + self.menke.bornSkill[self.sid] * level) / 100
end

function gameDataManager.setNewGuestList(_data)
    if gameDataManager.newGuestList == nil then
        gameDataManager.newGuestList = {}
    end
    if t_menke[_data.sid].isShow == 1 then
        table.insert(gameDataManager.newGuestList, _data)
    end
end

function gameDataManager.getNewGuestList()
    if gameDataManager.newGuestList == nil then
        gameDataManager.newGuestList = {}
    end
    return gameDataManager.newGuestList
end

function gameDataManager.removeShowedGuestItem()
    if gameDataManager.newGuestList ~= nil then
        if #gameDataManager.newGuestList > 0 then
            table.remove(gameDataManager.newGuestList, 1)
        end
    end
end