--region ui基类
--Date
--此文件由[BabeLua]插件自动生成

require "common/class"

_EUIState = 
{
    Close = 1,
    Open = 2,
    Loading = 3,
}

_UICom = 
{
    Image = "Image",
    Button = "Button",
    Text = "Text",
    InputField = "InputField",
    Toggle = "Toggle",
    Slider = "Slider",
    UIList = "UIList",
    CanvasGroup = "CanvasGroup",
    RectTransform = "RectTransform",
    ScrollRect = "ScrollRect",
    RawImage = "RawImage",
}

UIBase = Class()

function UIBase:ctor()
    self.uiObject = nil
    self.uiName = ""
    self.uiState = _EUIState.Close
    self.script = nil
    self.unloadAbList = {}
	self.eventIDList = {}
    self.resCacheList = {}
    self.childPanelList = {}

    --[[@Liukeming 2018-1-5
        窗口创建后不会立即调start，而是从CUIBase.cs调用，所以
        我在CUIBase.cs start之后加了一个afterStart函数，默认
        会设置_started这个值，表明C#那边已经调用了Start

        新手引导会用到这个变量来判断所属窗口是否已经初始化并显示，否则会在delay阶段进行等待
    ]]
    self._started = false
end

function UIBase:getCom(childPath, uiCom, parent)
    local trans = parent and parent.transform or self.uiObject.transform
    local childObj = trans:Find(childPath)
	
	if not childObj then
		_gameTools.luaError('找不到节点' .. childPath)
		return nil
	end
	
    local _com = childObj:GetComponent(uiCom)
	
	if not _com then
		_gameTools.luaError('找不到组件' .. childPath .. '->' .. uiCom)
		return nil
	end
	
    return _com
end

function UIBase:getObj(childPath, parent)
    local trans = parent and parent.transform or self.uiObject.transform
    local childObj = trans:Find(childPath)

	if not childObj then
		_gameTools.luaError('找不到节点' .. childPath)
		return nil
	end
	
    return childObj.gameObject
end

function UIBase:addClickByPath(childPath, func, parent)
    local btn = self:getObj(childPath, parent)
    self:addClick(btn, func)
    return btn
end

function UIBase:addClick(go, func)
    --再加一层给引导用
    self.script:AddClick(go, function(...)
        local uiInfo = uiMgr:getUI(_UI.Guide)
        uiInfo:onButtonClickBef(...)
        func(...)
        uiInfo:onButtonClickAft(...)
    end, self)
end

function UIBase:addSliderValueChange(go, func)
    self.script:AddSliderOnValueChanged(go, func, self)
end

function UIBase:addToggleValueChange(go, func)
    self.script:AddToggleOnVlaueChanged(go, func, self)
end

--add by Liukeming 2017-12-7，添加移除事件
function UIBase:removeClick(go)
    if self.script then
        self.script:RemoveClick(go)
    end
end

function UIBase:refreshShader(obj)
    local ps = obj:GetComponent('UIParticleSystem')
    if(ps ~= nil) then
        if(ps.material ~= nil) then
            if(ps.material.name ~= 'Default UI Material') then
                ps.material.shader = UnityEngine.Shader.Find('UI/Additive')
            end
        end
    end
    for i = 1, obj.transform.childCount, 1 do
        self:refreshShader(obj.transform:GetChild(i - 1).gameObject)
    end
end
function UIBase:loadEffect(parentNode, effectName, destroyTime)
    local prefab = self:loadObject('eff_'..effectName, effectName)
    if(prefab == nil) then
        _gameTools.luaError('找不到特效' .. effectName)
    end
    local go = GameObject.Instantiate(prefab)
    self:refreshShader(go)
    go.transform:SetParent(parentNode.transform)
    go.transform.localPosition = Vector3.zero
    go.transform.localScale = Vector3.one
    if(destroyTime >= 0) then
        GameObject.Destroy(go, destroyTime)
    end
    return go
end

function UIBase:loadGunObj(resName)
    local prefab = self:loadObject("unit_"..resName, resName)
    if prefab == nil then
        _gameTools.luaError("找不到资源"..resName)
    end
    local go = GameObject.Instantiate(prefab)
    return go
end

--加载资源 1.ab的名字 2.资源的名字
function UIBase:loadObject(abName, resName)
    if self.resCacheList[abName] ~= nil then
        if self.resCacheList[abName][resName] ~= nil then
            return self.resCacheList[abName][resName]
        else
            self.resCacheList[abName][resName] = resMgr:loadObject(abName, resName)
            self.unloadAbList[abName] = self.unloadAbList[abName] + 1
            return self.resCacheList[abName][resName]
        end
    end

    if self.unloadAbList[abName] == nil then
        self.unloadAbList[abName] = 0
    else
        self.unloadAbList[abName] = self.unloadAbList[abName] + 1
    end

    if self.resCacheList[abName] == nil then
        self.resCacheList[abName] = {}
    end
    self.resCacheList[abName][resName] = resMgr:loadObject(abName, resName)
    return self.resCacheList[abName][resName]
end

--加载大图资源
function UIBase:loadLargeImg(resName)
    return CGameUtil.TransToSprite(self:loadObject(gameDef.ResImagePrefix..resName, resName))
    --return self:loadObject(abName, resName)
end

--动画显示
function UIBase:scaleToOpen(nodeRoot, callback)
    local wnd = self
    local seq = DG.Tweening.DOTween.Sequence()
    BridgeToLua.TweenTo(seq, 0, 1, 0.1, function(v)
        nodeRoot.transform.localScale = Vector3.one * v
    end)
    BridgeToLua.TweenCallback(seq, function()
        if callback then
            callback(wnd)     
        end 
    end)
end

--动画关闭
function UIBase:scaleToClose(nodeRoot, callback)
    local wnd = self
    local seq = DG.Tweening.DOTween.Sequence()
    BridgeToLua.TweenTo(seq, 1, 0, 0.1, function(v)
        nodeRoot.transform.localScale = Vector3.one * v
    end)
    BridgeToLua.TweenCallback(seq, function()
        if callback then
            callback(wnd)  
        end
        wnd:closeSelf()
    end) 
end

--关闭自己
function UIBase:closeSelf()
    uiMgr:closeUI(self.uiName)  
end

--加载子面板
function UIBase:loadChildPanel(uiChildPanel, ...)
    local abName = gameDef.ResUIPrefix .. string.lower(uiChildPanel.resName)
    local uiGo = GameObject.Instantiate(resMgr:loadObject(abName, uiChildPanel.resName))
    uiGo.transform:SetParent(self.uiObject.transform)
    uiGo.transform.localScale = Vector3(1,1,1)
    uiGo.transform.localPosition = Vector3(0,0,0)
    local uiLua = uiChildPanel.lua.new()
    local uiCom = uiGo:AddComponent(typeof(CUIBase))
    uiLua.uiObject = uiGo
    uiLua.uiState = _EUIState.Open
    uiLua.script = uiCom
    uiCom.luaTab = uiLua
    uiCom:SetOpenCallArgs(unpack({...}))
    self.childPanelList[uiLua] = uiChildPanel
    return uiLua
end

function UIBase:registerEvent(eventKey, obj, callBackEvent, ...)
	if not eventKey then
		error('注册失败，事件ID不存在：' .. tostring(eventKey))
	end

	local uid = eventMgr:registerEvent(eventKey, callBackEvent, obj, ...)
	table.insert(self.eventIDList, uid)
end

function UIBase:close()
    for k, v in pairs(self.childPanelList) do
        k:close()
        resMgr:unloadAssetBundle(gameDef.ResUIPrefix..string.lower(v.resName))
    end

    --卸载此界面加载的资源
    for k, v in pairs(self.unloadAbList) do
        for i = 1, v do
            resMgr:unloadAssetBundle(k)
        end
    end
    self.unloadAbList = {}
    self.resCacheList = {}

    self.childPanelList = {}
	
	--卸载注册的事件
	for i,v in pairs(self.eventIDList) do
		eventMgr:remByUniqueId(v)
	end
    self.eventIDList = {}
	
    GameObject.Destroy(self.uiObject)
    self.uiState = _EUIState.Close
    self.uiObject = nil
    self.script = nil

    if self.onClear ~= nil then
        self:onClear()
    end
    
    --@Liukeming 2018-1-5 关闭窗口先设置标记，以便下层的窗口触发引导
    self._started = false
end

function UIBase:afterStart()
    self._started = true
    --@Liukeming 2018-1-5 窗口正在打开，能看到的时候检查引导
    gameDataManager.guideData:checkActivable()
end

--endregion
